//@flow
import { extendObservable, observable, action, computed } from 'mobx';
import type { Message, Chat } from '../domain/Entities';
import * as Server from '../api/Server';
import UserStore from '../user/UserStore';


class ChatStore {
    chats: Array<Chat>;
    unreadChats: number;
    hasUnreadChats: boolean;
    addChat: (chat: Chat) => void;
    markAsRead: (chat: Chat) => void;
    addMessage: (chat: Chat, message: Message) => void;


    constructor() {
        extendObservable(this, {
            chats: [],
            addChat: action((chat: Chat) => {
                this.chats.push(chat);
            }),
            addMessage: action((chat: Chat, message: Message) => {
                chat.unread = true;
                chat.messages.push(message);
            }),
            markAsRead: action((chat: Chat) => {
                chat.unread = false;
            }),
            unreadChats: computed(() => {
                return this.chats.filter(chat => chat.unread).length;
            }),
            hasUnreadChats: computed(() => {
                return this.unreadChats > 0;
            })
        })
    }

    subscribeToChatMessages() {
        Server.subscribeToChatMessages((nextMessage: Message) => {
            const chatForMessage = this.chats.find(chat => chat.id === nextMessage.user);
            if (chatForMessage) {
                this.addMessage(chatForMessage, nextMessage);
            } else {
                const user = UserStore.users.find(user => user.id === nextMessage.user);
                if (user) {
                    const newChat = {
                        id: nextMessage.user,
                        user: user,
                        messages: [nextMessage],
                        unread: true
                    };
                    this.addChat(newChat);
                }
            }
        });
    };

    unsubscribeChatMessages() {
        Server.unsubscribeChat();
    };

}

const chatStore = new ChatStore();
export default chatStore;